package modele.info1.ships;
import java.lang.*;
import  info1.ships.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * une implémentation "abstraite" d'un bateau quelconque, de taille indéterminée
 */


public abstract class Ship implements IShip {

    private String name;
    Coord cord1;
    Coord cord2;


    /**
     * NB : LA SIGNATURE DU CONSTRUCTEUR DOIT ETRE RESPECTEE
     *
     * construit un bateau quelconque
     * @param name le nom du bateau
     * @param ayFront la coordonnée de la proue du bateau
     * @param ayBack la coordonnée de la poupe du bateau
     * @throws BadCoordException si l'une des coordonnées données ne définit pas une coordonnée alphanumérique correcte
     * @throws CoordsBadShipException si les coordonnées données ne permettent pas de définir un bateau correct :
     *  une ligne, une colonne, de la bonne taille, etc.
     */
    public Ship(String name, String ayFront, String ayBack)
            throws BadCoordException, CoordsBadShipException {
        this.name=name;
        this.cord1=new Coord(ayFront);
        this.cord2=new Coord(ayBack);
        if(getCategory().getSize()!=getSize()){
            throw new CoordsBadShipException();
        }
    }


    @Override
    public List<ICoord> getCoords() {
        ArrayList<ICoord> list = new ArrayList<>();
        Coord icord1 = cord1;
        Coord icord2 = cord2;
        int j = 0;
        if(cord1.toString().equals(cord2.toString())){
            list.add(cord1);
        } else {
            if(cord1.getY()==cord2.getY()){
                for(int i =0;i<getSize();i++){
                    char c;
                    if(cord1.alphax>cord2.alphax){
                        c = (char) (icord1.alphax - i);
                    } else {
                        c = (char) (icord1.alphax + i);
                    }
                    try { list.add(new Coord(c+icord1.y)); } catch (BadCoordException e) { e.printStackTrace();}
                }
            }
            if(cord1.getX()==cord2.getX()){
                for(int i =0;i<getSize();i++){
                    if(cord1.getY()>cord2.getY()){
                        String c = String.valueOf(icord1.getY()-i);
                        try { list.add(new Coord(icord1.alphax+c)); } catch (BadCoordException e) {e.printStackTrace();}
                    } else {
                        String c = String.valueOf(icord1.getY()+i);
                        try { list.add(new Coord(icord1.alphax+c)); } catch (BadCoordException e) {e.printStackTrace();}
                    }
                }
            }
        }
        return list;
    }

    @Override
    public ICoord getFront() {
        return cord1;
    }

    @Override
    public ICoord getBack() {
        return cord2;
    }

    @Override
    public String getName() {
        return this.name;
    }

    @Override
    public int getSize() {
        if(cord1.getY()==cord2.getY()){
            if(cord1.getX()>cord2.getX())
                return cord1.getX()-cord2.getX()+1;
            return cord2.getX()-cord1.getX()+1;
        }
        if(cord1.getX()==cord2.getX()){
            if(cord1.getY()>cord2.getY())
                return cord1.getY()-cord2.getY()+1;
            return cord2.getY()-cord1.getY()+1;
        }
        return -1;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Ship ship = (Ship) o;
        return Objects.equals(name, ship.name) && Objects.equals(cord1, ship.cord1) && Objects.equals(cord2, ship.cord2);
    }

    @Override
    public String toString() {
        return "Ship{" +
                "name='" + name + '\'' +
                ", cord1=" + cord1 +
                ", cord2=" + cord2 +
                '}';
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, cord1, cord2);
    }
}
